# coding: UTF-8
#_checkCharDesc.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012 Takuya Nishimoto (NVDA Japanese Team)

from __future__ import print_function

# CONSOLE_ENCODING = 'utf-8' # mintty
CONSOLE_ENCODING = 'cp932' # cmd.exe

LOCALE_JA = r'..\source\locale\ja'
SY_FILE = LOCALE_JA + r'\symbols.dic'
CH_FILE = LOCALE_JA + r'\characterDescriptions.dic'
CS_FILE = LOCALE_JA + r'\characters.dic'

import re

def my_print(s):
	print(s.encode(CONSOLE_ENCODING))

def read_symbol_file(sy_file):
	with open(sy_file) as sy:
		mode = None
		ar = {}
		c = 0
		for line in sy:
			c += 1
			line = line.rstrip().decode('utf-8')
			if line == 'complexSymbols:': 
				mode = 1
			    # print c, line
				continue
			if line == 'symbols:': 
				mode = 2
			    # print c, line
				continue
			if len(line) == 0: continue
			if line[0] == '#': continue
			if line[0:2] == '\\#': 
				line = '#' + line[2:]
			if line[0:2] == '\\0': 
				continue #line = unichr(0) + line[2:]
			if mode == 2:
				a = line.split('\t')
				if len(a) >= 2:
					if len(a[0]) != 1: continue
					#my_print("%d %s %s" % (c, a[0], a[1]))
					ar[a[0]] = [c, a[1]]
	return ar

def read_chardesc_file(ch_file):
	with open(ch_file) as ch:
		ar = {}
		c = 0
		for line in ch:
			c += 1
			line = line.rstrip().decode('utf-8')
			if len(line) == 0: continue
			if line[0] == '#': continue
			if line[0:2] == '\\#': 
				line = '#' + line[2:]
			a = line.split('\t')
			if len(a) >= 2:
				#my_print("%d %s %s" % (c, a[0], a[1]))
				ar[a[0]] = [c, a[1]]
	return ar

def read_characters_file(cs_file):
	with open(cs_file) as ch:
		ar = {}
		c = 0
		for line in ch:
			c += 1
			line = line.rstrip().decode('utf-8')
			if len(line) == 0: continue
			if line[0] == '#': continue
			if line[0:2] == '\\#': 
				line = '#' + line[2:]
			a = line.split('\t')
			if len(a) >= 4:
				ar[a[0]] = [c, a[3]]
	return ar

def print_diff(sy, ch):
	for k,v in ch.items():
		if k in sy:
			if v[1] == sy[k][1]: continue
			my_print("ch %d %s %s / sy %d %s" % (v[0], k, v[1], sy[k][0], sy[k][1]))

C = re.compile(u'\s+')
def equals_ignore_spaces(s1, s2):
	s1 = C.sub('', s1)
	s2 = C.sub('', s2)
	if s1 == s2: return True
	return False
	
def print_diff_ignore_spaces(sy, ch):
	ar = {}
	for k,v in ch.items():
		if k in sy:
			s1 = v[1]
			s2 = sy[k][1]
			if equals_ignore_spaces(s1, s2): continue
			ar[sy[k][0]] = "%04x sy %d %s / ch %d %s %s" % (ord(k), sy[k][0], sy[k][1], v[0], k, v[1])
	for s in sorted(ar.items(), key=lambda x:int(x[0])):
		my_print(s[1])
		
def find_desc_duplicate(ch):
	for k,v in ch.items():
		for k2,v2 in ch.items():
			if v[0] < v2[0] and k != k2 and equals_ignore_spaces(v[1], v2[1]):
				my_print("ch %d:%s %04x / %d:%s %04x / %s" % (v[0], k, ord(k), v2[0], k2, ord(k2), v2[1]))

def isZenkakuKatakana(c):
	return re.search(ur'[ァ-ヾ]', c) is not None

def isHankakuKatakana(c):
	return re.search(ur'[ｦ-ﾝ]', c) is not None

def isHalfShape(c):
	c = c[0:1]
	return (32 < ord(c)) and (ord(c) < 128)

def add_katakana_prefix_to_characters(ch):
	ar = {}
	for k,v in ch.items():
		if isZenkakuKatakana(k):
			v = u'カタカナ ' + unicode(v)
		elif isHankakuKatakana(k):
			v = u'ハンカクカタカナ ' + unicode(v)
		elif k.isupper():
			v = u'オオモジ ' + unicode(v)
		elif isHalfShape(k):
			v = u'ハンカク ' + unicode(v)
		ar[k] = v
	return ar
