# coding: UTF-8
#evaluate.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2012 Takuya Nishimoto (NVDA Japanese Team)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
# 
# Usage for Test:
# > cd miscdep\include\jtalk
# > all-build
# > all-install
# > python evaluate.py > output-utf8.txt

import sys
sys.path.append(r'..\source\synthDrivers\jtalk')
from translator2 import *
import os

# 数 283C   3456 ⠼ 数符
# .         2      小数点
# '         3      位取り点、アポストロフィ
#
# 外 2830   56   ⠰ 外字符
# 大 2820   6    ⠠ 大文字符
# ＜ 2826   236  ⠦ 外国語引用符 全角小なり
# ＞ 2834   356  ⠴ 外国語引用符(とじる) 全角大なり
#
# 。        256-0-0 句点
# 、        56-0    読点
# ・        5(2?)   中点
# ？        26-0-0  疑問符
# ！        235-0-0 感嘆符
# － 2824   36   ⠤ 第一つなぎ符 全角マイナス
#           6-3     第二つなぎ符
#
# 「 2824   36   ⠤ 第一かぎ
# 」 2824   36   ⠤ 第一かぎ閉じる
# 『        56-36   第二かぎ
# 』        36-23   第二かぎ閉じる
# （        2356    第一カッコ
# ）        2356    第一カッコ閉じる
#           5-2356  第二カッコ
#           2356-2  第二カッコ閉じる
#           56-2356 二重カッコ
#           2356-23 二重カッコ閉じる
#
#           25-25   棒線
#           2-2-2   点線
#           25-25-135  右矢印
#           246-25-25  左矢印
#           246-25-25-135  両向き矢印
# ～        36-36   波線
# ％        56-1234 パーセント(pと同じ)
# ＆        56-12346 アンドマーク
# ＃        56-146   ナンバーマーク
# ＊        56-16    アステリスク
#
# 《        6-236    情報処理用点字
# 》        6-356    情報処理用点字閉じる
#           4        行継続符
# :         5-2      情報処理用点字 コロン
# /         34       情報処理用点字 スラッシュ
# @         246      情報処理用点字 アットマーク
# _         5-36     情報処理用点字 アンダーライン
# ~         5-14     情報処理用点字 オーバーライン、チルダ

tests = [
	[u"NVDA 2010.2 ユーザーガイド", u"外大大nvda 数2010. 数2 ユーザー ガイド"],
	[u"一般的な機能", u"イッパンテキナ キノー"],
	[u"国際化", u"コクサイカ"],
	[u"音声合成サポート", u"オンセイ ゴーセイ サポート"],
	[u"点字ディスプレイサポート", u"テンジ ディスプレイ サポート"],
	[u"ライセンスと著作権", u"ライセンスト チョサクケン"],
	[u"システム要件", u"システム ヨーケン"],
	[u"NVDAの入手", u"外大大nvda ノ ニューシュ"],
	[u"NVDA入門", u"外大大nvda ニューモン"],
	[u"NVDAの起動", u"外大大nvda ノ キドー"],
	[u"コマンドについて", u"コマンドニ ツイテ"],
	[u"フォーカスによる移動", u"フォーカスニ ヨル イドー"],
	[u"システムキャレット", u"システム キャレット"],
	[u"システムキャレットによる移動", u"システム キャレットニ ヨル イドー"],
	[u"オブジェクト内の移動", u"オブジェクトナイノ イドー"],
	[u"現在オブジェクトのテキストの確認", u"ゲンザイ オブジェクトノ テキストノ カクニン"],
	[u"マウスによる移動", u"マウスニ ヨル イドー"],
	[u"仮想バッファー", u"カソー バッファー"],
	[u"単一文字によるナビゲーション", u"タンイツ モジニ ヨル ナビゲーション"],
	[u"フォームとフォーカスモード", u"フォームト フォーカス モード"],
	[u"要素リスト", u"ヨーソ リスト"],
	[u"埋め込みオブジェクト", u"ウメコミ オブジェクト"],
	[u"設定メニュー", u"セッテイ メニュー"],
	[u"一般設定", u"イッパン セッテイ"],
	[u"出力先の切り替え", u"シュツリョクサキノ キリカエ"],
	[u"音声設定", u"オンセイ セッテイ"],
	[u"点字設定", u"テンジ セッテイ"],
	[u"読み上げ辞書", u"ヨミアゲ ジショ"],
	[u"さらに詳細な情報", u"サラニ ショーサイナ ジョーホー"],
	[u"NVDA 2010.2 User Guideの日本語訳です。", u"＜大大nvda 数2010. 数2 大user 大guide＞ ノ ニホンゴヤクデス。  "],
	[u"正式な文書ではないことにご注意下さい。", u"セイシキナ ブンショデワ ナイ コトニ ゴチューイ クダサイ。  "],
	[u"また、翻訳において生じた誤りが含まれる可能性があります。", u"マタ、 ホンヤクニ オイテ ショージタ アヤマリガ フクマレル カノーセイガ アリマス。  "],
	[u"合成音声によるフィードバックによって、", u"ゴーセイ オンセイニ ヨル フィード バックニ ヨッテ、 "],
	[u"全盲者をはじめとした視覚障害者が", u"ゼンモーシャヲ ハジメト シタ シカク ショーガイシャガ"],
	[u"オペレーティングシステムや、さまざまなサードパーティー製のアプリケーションを利用することを可能にします。", u"オペレーティング システムヤ、 サマザマナ サード パーティーセイノ アプリケーションヲ リヨー スル コトヲ カノーニ シマス。  "],
	[u"主なハイライトは以下の通りです。", u"オモナ ハイライトワ イカノ トオリデス。  "],
	[u"インストールなしでUSBメモリースティックや他の携帯型メディアによる実行が可能", u"インストール ナシデ 外大大usb メモリー スティックヤ ホカノ ケイタイガタ メディアニ ヨル ジッコーガ カノー"],
	[u"簡単に利用できるトーキングインストーラー", u"カンタンニ リヨー デキル トーキング インストーラー"],
	[u"アクセシブルなJavaアプリケーションのサポート", u"アクセシブルナ ＜大java＞ アプリケーションノ サポート"],
	[u"Adobe Readerのサポート", u"＜大adobe 大reader＞ ノ サポート"],
	[u"マウス位置のテキストの自動読み上げおよびマウス位置をビープ音で知らせるオプション", u"マウス イチノ テキストノ ジドー ヨミアゲ オヨビ マウス イチヲ ビープオンデ シラセル オプション"],
	[u"多くの点字ディスプレイのサポート", u"オオクノ テンジ ディスプレイノ サポート"],
	[u"日本語版は未対応", u"ニホンゴバンワ ミタイオー"],
	[u"世界中の誰もが、どんな言語を話しているかに関わらず、等しくテクノロジーにアクセスできるということが重要です。", u"セカイジューノ ダレモガ、 ドンナ ゲンゴヲ ハナシテ イルカニ カカワラズ、 ヒトシク テクノロジーニ アクセス デキルト イウ コトガ ジューヨーデス。  "],
	[u"NVDAは現在、以下の言語を含む20ヶ国語以上の言語に翻訳されています。", u"外大大nvda ワ ゲンザイ、 イカノ ゲンゴヲ フクム 数20カコクゴ イジョーノ ゲンゴニ ホンヤク サレテ イマス。  "],
	[u"メッセージやインターフェースを複数の言語で提供できるだけでなく、もし利用者が使用したい言語に対応した音声エンジンを持っていれば、NVDAではどんな言語のコンテンツも読ませることができます。", u"メッセージヤ インター フェースヲ フクスーノ ゲンゴデ テイキョー デキルダケデ ナク、 モシ リヨーシャガ シヨー シタイ ゲンゴニ タイオー シタ オンセイ エンジンヲ モッテ イレバ、 外大大nvda デワ ドンナ ゲンゴノ コンテンツモ ヨマセル コトガ デキマス。  "],
	[u"NVDAは、無料でオープンソースの多言語対応音声合成ソフトウェアであるeSpeakをバンドルしています。", u"外大大nvda ワ、 ムリョーデ オープン ソースノ タゲンゴ タイオー オンセイ ゴーセイ ソフトウェアデ アル ＜e大speak＞ ヲ バンドル シテ イマス。  "],
	[u"加えてNVDAでは音声読み上げを行うためにSAPI4とSAPI5の両方の音声エンジンが利用できる他、Audiologic speech synthesizerやNewFon speech synthesizerも利用できます。", u"クワエテ 外大大nvda デワ オンセイ ヨミアゲヲ オコナウ タメニ 外大大sapi数4ト 外大大sapi数5ノ リョーホーノ オンセイ エンジンガ リヨー デキル ホカ、 ＜大audiologic speech synthesizer＞ヤ ＜大new大fon speech synthesizer＞モ リヨー デキマス。  "],
	[u"点字ディスプレイを所有している利用者に対して、NVDAは情報を点字で出力することができます。", u"テンジ ディスプレイヲ ショユー シテ イル リヨーシャニ タイシテ、 外大大nvda ワ ジョーホーヲ テンジデ シュツリョク スル コトガ デキマス。  "],
	[u"NVDAはFreedom Scientific社製のPacmateとFocus点字ディスプレイをサポートしている他、Handy Tech社製のディスプレイ、ALVA BC 640/680ディスプレイ、BRLTTYパッケージでサポートされている多くの点字ディスプレイに対応しています。", u"外大大nvda ワ ＜大freedom 大scientific＞－シャセイノ ＜大pacmate＞ ト ＜大focus＞ テンジ ディスプレイヲ サポート シテ イル ホカ、 ＜大handy 大tech＞－シャセイノ ディスプレイ、 外大大alva 外大大bc 数640・数680 ディスプレイ、 外大大brltty パッケージデ サポート サレテ イル オオクノ テンジ ディスプレイニ タイオー シテ イマス。  "],
	[u"NVDAでは、多くの点字コードとさまざまな言語をサポートしています。", u"外大大nvda デワ、 オオクノ テンジ コードト サマザマナ ゲンゴヲ サポート シテ イマス。  "],
	[u"多くの場合、1級と2級のコードが利用可能です。英語では、アメリカ英語点字とUnified English Braille(UEB)の両方が含まれています。", u"オオクノ バアイ、 数1キュート 数2キューノ コードガ リヨー カノーデス。  エイゴデワ、 アメリカ エイゴ テンジト ＜大unified 大english 大braille＞ (外大大ueb)ノ リョーホーガ フクマレテ イマス。  "],
	[u"NVDAはGNU一般公衆利用許諾契約書(Version 2)によって保護されています。", u"外大大nvda ワ 外大大gnu イッパン コーシュー リヨー キョダク ケイヤクショ(＜大version 数2＞)ニ ヨッテ ホゴ サレテ イマス。  "],
	[u"この内容に同意いただき、必要としている方にソースコードを提供していただける場合、本ソフトウェアの配布や改良は自由に行うことができます。", u"コノ ナイヨーニ ドーイ イタダキ、 ヒツヨート シテ イル カタニ ソース コードヲ テイキョー シテ イタダケル バアイ、 ホン ソフトウェアノ ハイフヤ カイリョーワ ジユーニ オコナウ コトガ デキマス。  "],
	[u"この規定は、オリジナルのソフトウェアと改良版を含むソフトウェアのコピー、さらにこのソフトウェアのコードを利用したソフトウェアにも適用されます。", u"コノ キテイワ、 オリジナルノ ソフトウェアト カイリョーバンヲ フクム ソフトウェアノ コピー、 サラニ コノ ソフトウェアノ コードヲ リヨー シタ ソフトウェアニモ テキヨー サレマス。  "],
	[u"本ライセンスについての詳細は、オンラインで確認してください。", u"ホン ライセンスニ ツイテノ ショーサイワ、 オンラインデ カクニン シテ クダサイ。  "],
#	[u"", u""],
	[u"日本語版に関しては、 http://sourceforge.jp/projects/nvdajp/ のダウンロードページに最新のパッケージがあります。", u"ニホンゴバンニ カンシテワ、 《http://sourceforge.jp/projects/nvdajp/》 ノ ダウン ロード ページニ サイシンオ パッケージガ アリマス。  "],
#	[u"", u""],
	[u"テンキー + （プラス）", u"テンキー 《+》 (プラス)"],
	[u"テンキー / （スラッシュ）", u"テンキー 《/》 (スラッシュ)"],
	[u"テンキー * （アステリスク）", u"テンキー 《*》 (アステリスク)"],
	[u"Shift + テンキー * （アステリスク）", u"＜大shift＞ プラス テンキー 《*》 (アステリスク)"],
	# 第一カッコによる記号の読み下し
	[u"●", u"（クロマル）"],
	[u"底辺×高さ", u"テイヘン（カケル）タカサ"],
#	[u"", u""],
	[u"スタートメニュー「プログラム」にある", u"スタート メニュー 「プログラム」ニ アル"],
	[u"広い海を泳ぐ小さい魚。", u"ヒロイ ウミヲ オヨグ チーサイ サカナ。  "],
	[u"美しい自然。", u"ウツクシイ シゼン。  "],
	[u"百円ショップは何でも１００円。", u"数100－エン ショップワ ナンデモ 数100－エン。  "],
	[u"ねえ、先生。", u"ネエ、 センセイ。  "],
	[u"母は花屋へ行きました。", u"ハハワ ハナヤエ イキマシタ。  "],
	[u"字を書く", u"ジヲ カク"],
	[u"お母さん", u"オカアサン"],
	[u"映画", u"エイガ"],
	[u"空気", u"クーキ"],
	[u"お父さん", u"オトーサン"],
	[u"閏年", u"ウルウドシ"],
	[u"思う", u"オモウ"],
	[u"ありがとう", u"アリガトー"],
	[u"狼", u"オオカミ"],
	[u"通る", u"トオル"],
	[u"多い", u"オオイ"],
	[u"大晦日", u"オオミソカ"],
	[u"手作り", u"テヅクリ"],
	[u"インタビュー", u"インタビュー"],
	[u"日本点字図書館", u"ニッポン テンジ トショカン"],
	[u"一円硬貨", u"数1－エン コーカ"],
	[u"10メートル四方", u"数10メートル 数4ホー"],
	[u"十メートル四方", u"数10メートル 数4ホー"],
	[u"百人一首", u"数100ニン 数1シュ"],
	[u"十七歳", u"数17サイ"],
	[u"17歳", u"数17サイ"],
	[u"五つ星", u"イツツボシ"],
	[u"三、四日", u"数3数4カ"],
	[u"ボーイング777", u"ボーイング 数777"],
	[u"徒歩で五分", u"トホデ 数5フン"],
	[u"500kHz", u"数500外k大hz"],
	[u"AbCD", u"外大ab大大cd"],
	[u"二十五度C", u"数25ド外大c"],
	[u"JR線", u"外大大jr－セン"],
	[u"逆V字型", u"ギャク外大v－ジガタ"],
	[u"TOEFLテスト", u"外大大toefl テスト"],
	[u"C-90", u"外大c－数90"],
	[u"新聞や雑誌などなど", u"シンブンヤ ザッシナドナド"],
	[u"今日は晴れるでしょう", u"キョーワ ハレルデショー"],
	[u"50へ手が届く", u"数50－エ テガ トドク"],
	[u"見てる", u"ミテル"],
	[u"見ている", u"ミテ イル"],
	[u"歩き始める", u"アルキハジメル"],
	[u"理論的すぎる", u"リロンテキスギル"],
	[u"利用できる", u"リヨー デキル"],
	[u"運動をする", u"ウンドーヲ スル"],
	[u"意図する", u"イト スル"],
	[u"読書しない", u"ドクショ シナイ"],
	[u"面白おかしい", u"オモシロオカシイ"],
	[u"赤くなる", u"アカク ナル"],
	[u"問題ない", u"モンダイ ナイ"],
	[u"山の手", u"ヤマノテ"],
	[u"受付窓口", u"ウケツケ マドグチ"],
	[u"六法全書", u"数6ポー ゼンショ"],
	[u"アイスクリーム", u"アイス クリーム"],
	[u"仮名文字", u"カナモジ"],
	[u"南半球", u"ミナミ ハンキュー"],
	[u"北半球", u"キタハンキュー"],
	[u"事務機器", u"ジム キキ"],
	[u"右半分", u"ミギ ハンブン"],
	[u"株式会社", u"カブシキガイシャ"],
	[u"小中学校", u"ショーチューガッコー"],
	[u"都道府県", u"トドーフケン"],
	[u"二〇〇〇年一月一日", u"数2000ネン 数1ガツ ツイタチ"],
	[u"二千年一月一日", u"数2000ネン 数1ガツ ツイタチ"],
	[u"ルイ・ブライユ", u"ルイ ブライユ"],
	[u"アメリカ合衆国", u"アメリカ ガッシューコク"],
	[u"小一時間", u"コ数1ジカン"],
	[u"第一人者", u"ダイ数1ニンシャ"],
	[u"全五巻", u"ゼン数5カン"],
	[u"超一流", u"チョー 数1リュー"],
	[u"一人一人", u"ヒトリ ヒトリ"],
	[u"盲", u"モー"],
	[u"聾", u"ロー"],
	[u"かな文字", u"カナモジ"],
	[u"ウィークデー", u"ウィークデー"],
	[u"窓ガラス", u"マドガラス"],
	[u"松並木", u"マツナミキ"],
	[u"水遊び", u"ミズアソビ"],
	[u"バス停留所", u"バス テイリュージョ"],
]

def find_blocks(text):
	# find_blocks(u"ああああああaaaaaあああああbbbbbb あああ")
	# => [ (0,6,'ja'), (6,5,'en'), (11,5,'ja'), (16,6,'en'), (22,4,'ja') ]

	# eng_flags: 00000011111000001111110000
	eng_flags = []
	for i in xrange(0, len(text)):
		if is_alphabet(text[i]) or is_num_char(text[i]) or text[i] == ' ' or text[i] == '.':
			eng_flags.append(1)
		else:
			eng_flags.append(0)
	
	# trim space on the edge
	for i in xrange(0, len(eng_flags) - 1):
		if text[i] == ' ' and eng_flags[i+1] == 0:
			eng_flags[i] = 0
	
	#for i in xrange(0, len(text)): print eng_flags[i],
	#print

	# eng_len: 0000005432100000654321000
	eng_len = []
	for i in xrange(0, len(eng_flags)): eng_len.append(0)
	prev = False
	for i in reversed(xrange(0, len(eng_flags))):
		if eng_flags[i]:
			if prev:
				eng_len[i] = eng_len[i+1] + 1
			else:
				eng_len[i] = 1
			prev = True
		else:
			prev = False

	#for i in xrange(0, len(text)): print eng_len[i],
	#print

	# blocks = [ (6,5,'en'), (16,6,'en') ]
	blocks = []
	p = 0
	while p < len(text):
		if eng_len[p] >= 5:
			item = (p, eng_len[p], 'en', text[p:p+eng_len[p]])
			blocks.append(item)
			p += eng_len[p]
		else:
			p += 1

	# blocks2 = [ (0,6,'ja'), (6,5,'en'), (11,5,'ja'), (16,6,'en'), (22,3,'ja') ]
	blocks2 = []
	p = 0
	for a,b,c,d in blocks:
		if p < a:
			item = (p,a-p,'ja', text[p:a])
			blocks2.append(item)
		blocks2.append((a,b,c,d))
		p = a+b
	if p < len(text):
		blocks2.append((p, len(text)-p, 'ja', text[p:]))
	return blocks2

def insert_num_sign(text):
	s1 = ''
	in_num = False
	# 50エ -> 数50エ
	for c in text:
		if is_num_char(c):
			if not in_num:
				s1 += u'数'
			in_num = True
		else:
			in_num = False
		s1 += c
	return s1

def insert_english_quote(text):
	text = u'＜' + text + u'＞'
	return text

def insert_english_sign(text):
	in_eng = False
	s = ''
	for i in xrange(0, len(text)):
		c = text[i]
		c1 = ''
		if i+1 < len(text):
			c1 = text[i+1]
		if is_alphabet(c):
			if not in_eng:
				s += u'外'
			in_eng = True
		else:
			in_eng = False
		s += c
	return s

def insert_english_cap_sign(text):
	is_upper_group = False
	s = ''
	for i in xrange(0, len(text)):
		c = text[i]
		c1 = ''
		if i+1 < len(text):
			c1 = text[i+1]
		if is_alphabet(c):
			if is_uppercase(c) and not is_upper_group:
				if is_uppercase(c1):
					s += u'大大'
					is_upper_group = True
				else:
					s += u'大'
					is_upper_group = False
		else:
			is_upper_group = False
		s += c.lower()
	return s

def replace_symbol_before_num_sign(text):
	s = ''
	for i in xrange(0, len(text)):
		append_space = False
		c = text[i]
		c1 = ''
		if i+1 < len(text):
			c1 = text[i+1]
		if c1 == u'数':
			if c == '.':
				append_space = True
			elif c == '-':
				c = u'－'
		s += c
		if append_space:
			s += ' '
	return s

def separate_after_digits(text):
	# 数50エ -> 数50－エ
	s2 = ''
	prev_num = False
	for c in text:
		if prev_num and c in u'アイウエオ':
			s2 += u'－'
		s2 += c
		prev_num = is_num_char(c)
	return s2

def braille_translate(text, logwrite):
	blocks = find_blocks(text)
	s = ''
	for a,b,c,d in blocks:
		if c == 'ja':
			text = japanese_braille_separate(d, logwrite)
			text = insert_english_sign(text)
			text = insert_num_sign(text)
			text = insert_space_after_alphabet(text)
			text = insert_english_cap_sign(text)
			text = replace_symbol_before_num_sign(text)
			text = separate_after_digits(text)
			text = re.sub(u"、", u"、 ", text)
			text = re.sub(u"。", u"。  ", text)
			if a != 0: text = ' ' + text
			s += text
		elif c == 'en':
			text = insert_english_quote(d)
			text = insert_num_sign(text)
			text = insert_english_cap_sign(text)
			text = replace_symbol_before_num_sign(text)
			if a != 0: text = ' ' + text
			s += text
	s = re.sub(u" －", u"－", s)
	return s

def test_braille(jtalk_dir, logwrite):
	initialize(jtalk_dir, logwrite)
	count_ok = 0
	count_all = 0
	for i in tests:
		count_all += 1
		logwrite('')
		logwrite('Task %d' % count_all)
		logwrite('')
		s = braille_translate(i[0], logwrite)
		correct = 0
		if s == i[1]: 
			count_ok += 1
			correct = 1
		logwrite("  '%s'" % i[0])
		logwrite("  '%s'" % i[1])
		logwrite("%d '%s'" % (correct, s))
	logwrite('')
	logwrite("%d / %d passed" % (count_ok, count_all))

if __name__ == "__main__":
	logwrite = None
	try:
		from logHandler import log
		logwrite = log.info
	except:
		def __print(s): print s.encode('utf-8', 'ignore')
		logwrite = __print
	jtalk_dir = os.path.join(os.getcwdu(), '..', 'source', 'synthDrivers', 'jtalk')
	test_braille(jtalk_dir, logwrite)
	#print find_blocks(u"ああああああaaaaaあああああbbbbbb あああ")
