# -*- coding: utf-8 -*-
#jptools/harness.py
#A part of NonVisual Desktop Access (NVDA)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#Copyright (C) 2012 Masataka.Shinke, Takuya Nishimoto

import sys
sys.path.append(r'..\source\synthDrivers\jtalk')
from tenji import * 

outfile = '__h1output.txt'

# eBraille
# https://ebraille.med.kobe-u.ac.jp/eBraille2/
#
# Japanese information Braille
# http://homepage2.nifty.com/a_sato/tenji-room/fullkey/jyohou.html
#
# Historical changes of the Japanese mathematic Braille code
# http://www.tsukuba-tech.ac.jp/repo/dspace/handle/10460/541

tests = [
	{
		'input': u'アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨン',
		'output': u'⠁⠃⠉⠋⠊⠡⠣⠩⠫⠪⠱⠳⠹⠻⠺⠕⠗⠝⠟⠞⠅⠇⠍⠏⠎⠥⠧⠭⠯⠮⠵⠷⠽⠿⠾⠌⠬⠜⠴',
		'cursorPos': 0,
		'brlCursorPos': 0,
		'mode': '',
		'comment': '',
		},
	{
		'input': u'ガギグゲゴザジズゼゾダジズデドパピプペポキャキュキョチャチュチョヒャヒュヒョジャジュジョピャピュピョ',
		'output': u'⠐⠡⠐⠣⠐⠩⠐⠫⠐⠪⠐⠱⠐⠳⠐⠹⠐⠻⠐⠺⠐⠕⠐⠳⠐⠹⠐⠟⠐⠞⠠⠥⠠⠧⠠⠭⠠⠯⠠⠮⠈⠡⠈⠩⠈⠪⠈⠕⠈⠝⠈⠞⠈⠥⠈⠭⠈⠮⠘⠱⠘⠹⠘⠺⠨⠥⠨⠭⠨⠮',
		},
	{
		'input': u'1234567890',
		'output': u'⠼⠁⠃⠉⠙⠑⠋⠛⠓⠊⠚',
		},
	{
		'input': u'abcdefghijklmnopqrstuvwxyz',
		'output': u'⠰⠁⠃⠉⠙⠑⠋⠛⠓⠊⠚⠅⠇⠍⠝⠕⠏⠟⠗⠎⠞⠥⠧⠺⠭⠽⠵',
		},
	{
		'input': u'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
		'output': u'⠰⠠⠠⠁⠃⠉⠙⠑⠋⠛⠓⠊⠚⠅⠇⠍⠝⠕⠏⠟⠗⠎⠞⠥⠧⠺⠭⠽⠵',
		},
	{
		'input': u'(U)',
		'output': u'⠶⠰⠠⠥⠶',
		'comment': 'nvdajp ticket 28226 (comment 2012-05-16)',
		},
	{
		'input': u'（U）',
		'output': u'⠶⠰⠠⠥⠶',
		'comment': '',
		},
	{
		'input': u'[ア]',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'［ア］',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'“ア”',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'{ア}',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'‘ア’',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'〔ア〕',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'｛ア｝',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'〈ア〉',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'《ア》',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'【ア】',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'〝ア〟',
		'output': u'⠐⠶⠁⠶⠂',
		'comment': '',
		},
	{
		'input': u'「ア」',
		'output': u'⠰⠤⠁⠤⠆',
		'comment': '',
		},
	{
		'input': u'『ア』',
		'output': u'⠰⠤⠁⠤⠆',
		'comment': '',
		},
	{
		'input': u'｢ア｣',
		'output': u'⠰⠤⠁⠤⠆',
		'comment': '',
		},
	{
		'input': u'Ctrl-Z',
		'output': u'⠰⠠⠉⠞⠗⠇⠤⠰⠠⠵',
		'comment': 'nvdajp ticket 28226 (comment 2012-05-16)',
		},
	{
		'input': u'Aア',
		'output': u'⠰⠠⠁⠤⠁',
		},
	{
		'input': u'aアットb',
		'output': u'⠰⠁⠤⠁⠂⠞⠰⠃',
		},
	{
		'input': u'a@b',
		'output': u'⠰⠁⠤⠁⠂⠞⠰⠃',
		'comment': 'nvdajp ticket 28226 (comment 2012-05-16)',
		},
	{
		'input': u'Cプラスプラス',
		'output': u'⠰⠠⠉⠤⠠⠭⠑⠹⠠⠭⠑⠹',
		},
	{
		'input': u'notepad++',
		'output': u'⠰⠝⠕⠞⠑⠏⠁⠙⠤⠠⠭⠑⠹⠠⠭⠑⠹',
		'comment': 'nvdajp ticket 28226 (comment 2012-05-16)',
		},
	{
		'input': u'v1.4',
		'output': u'⠰⠧⠼⠁⠂⠼⠙',
		'comment': 'nvdajp ticket 28226 (comment 2012-05-16)',
		},
	{
		'input': u'c:\\',
		'output': u'⠰⠉⠐⠂⠡',
		'comment': 'the last character is backslash (path separator)',
		},
	{
		'input': u'アー',
		'output': u'⠁⠒',
		},
	{
		'input': u'ア、イ。',
		'output': u'⠁⠰⠃⠲',
		},
	{
		'input': u'ア？ア?',
		'output': u'⠁⠢⠁⠢',
		},
	{
		'input': u'ア！ア!',
		'output': u'⠁⠖⠁⠖',
		},
	{
		'input': u'ア・イ',
		'output': u'⠁⠐⠃',
		},
	{
		'input': u'123アイウ',
		'output': u'⠼⠁⠃⠉⠤⠁⠃⠉',
		'comment': 'numbers and kana characters should be separated by 36 symbol.',
		},
	{
		'input': u'0ア1イ2ウ3エ4オ5ラ6リ7ル8レ9ロ',
		'output': u'⠼⠚⠤⠁⠼⠁⠤⠃⠼⠃⠤⠉⠼⠉⠤⠋⠼⠙⠤⠊⠼⠑⠤⠑⠼⠋⠤⠓⠼⠛⠤⠙⠼⠓⠤⠛⠼⠊⠤⠚',
		'comment': 'numbers and some kana characters should be separated by 36 symbol.',
		},
	{
		'input': u'123キャ',
		'output': u'⠼⠁⠃⠉⠈⠡',
		'comment': 'should not be separated by 36 symbol.',
		},
	{
		'input': u'0カ1キ2ク3ケ4コ5サ6シ7ス8セ9ソ0タ1チ2ツ3テ4ト5ナ6ニ7ヌ8ネ9ノ',
		'output': u'⠼⠚⠡⠼⠁⠣⠼⠃⠩⠼⠉⠫⠼⠙⠪⠼⠑⠱⠼⠋⠳⠼⠛⠹⠼⠓⠻⠼⠊⠺⠼⠚⠕⠼⠁⠗⠼⠃⠝⠼⠉⠟⠼⠙⠞⠼⠑⠅⠼⠋⠇⠼⠛⠍⠼⠓⠏⠼⠊⠎',
		'comment': 'should not be separated by 36 symbol.',
		},
	{
		'input': u'1ア1キャ1ア',
		'output': u'⠼⠁⠤⠁⠼⠁⠈⠡⠼⠁⠤⠁',
		'comment': 'numbers and some kana characters should be separated by 36 symbol.',
		},
	{
		'input': u'123a',
		'output': u'⠼⠁⠃⠉⠰⠁',
		},
	{
		'input': u'123A',
		'output': u'⠼⠁⠃⠉⠰⠠⠁',
		},
	{
		'input': u'1アa1aキャA1Aア',
		'output': u'⠼⠁⠤⠁⠰⠁⠼⠁⠰⠁⠤⠈⠡⠰⠠⠁⠼⠁⠰⠠⠁⠤⠁',
		},
	{
		'input': u'⠀⠁⠂⠃⠄⠅⠆⠇⠈⠉⠊⠋⠌⠍⠎⠏⠐⠑⠒⠓⠔⠕⠖⠗⠘⠙⠚⠛⠜⠝⠞⠟',
		'output': u'⠀⠁⠂⠃⠄⠅⠆⠇⠈⠉⠊⠋⠌⠍⠎⠏⠐⠑⠒⠓⠔⠕⠖⠗⠘⠙⠚⠛⠜⠝⠞⠟',
		'comment': 'braille code should not be changed.',
		},
	{
		'input': u'⠠⠡⠢⠣⠤⠥⠦⠧⠨⠩⠪⠫⠬⠭⠮⠯⠰⠱⠲⠳⠴⠵⠶⠷⠸⠹⠺⠻⠼⠽⠾⠿',
		'output': u'⠠⠡⠢⠣⠤⠥⠦⠧⠨⠩⠪⠫⠬⠭⠮⠯⠰⠱⠲⠳⠴⠵⠶⠷⠸⠹⠺⠻⠼⠽⠾⠿',
		'comment': 'braille code should not be changed.',
		},
	{
		'text': u'《本章のねらい》',
		'input': u'《ホンショーノ ネライ》',
		# 'output': u'⠰⠄⠮⠴⠈⠺⠒⠎⠀⠏⠑⠃⠠⠆',
		'comment': u'56-3 6-23 なのか 56-2356 2356-23 なのか',
		},
	{
		'text': u'ピクチャ3の12',
		'input': u'ピクチャ3ノ 12',
		# 'output': u'',
		'comment': u'',
		},
	]

if __name__ == '__main__':
	with open(outfile, 'w') as f:
		count = 0
		for t in tests:
			if t.has_key('output') and tenji(t['input']) != t['output']:
				count+=1 
				f.write(t['input'].encode('utf-8') + "\n")
				f.write(tenji(t['input']).encode('utf-8') + "\n")
				f.write(t['output'].encode('utf-8') + "\n")
				if 'comment' in t:
					f.write(t['comment'].encode('utf-8') + "\n")
				f.write("\n")
		print '%d error(s). see %s' % (count, outfile)
