﻿/*
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 Takuya Nishimoto, Masataka Shinke (nvdajp)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
# nvdajpime/debugout.h
# nvdajpimeRPC/debugout.h
*/

#ifndef DEBUGOUT_H
#define DEBUGOUT_H

using namespace std;

/*
 * \HKEY_CURRENT_USER\Software\nvdajp -> DebugIME32 REG_DWORD 1
 */
inline DWORD GetDebugMode()
{
   HKEY hKey;
   if (RegOpenKeyEx( HKEY_CURRENT_USER, TEXT("SOFTWARE\\NVDAJP"), 0, KEY_WOW64_64KEY | KEY_READ, &hKey) != ERROR_SUCCESS) {
	   return 0;
   }
   DWORD type;
   if (RegQueryValueEx( hKey, TEXT("DebugIME32"), NULL, &type, NULL, NULL) != ERROR_SUCCESS) {
	   return 0;
   }
   if (type != REG_DWORD) {
	   return 0;
   }
   DWORD data;
   DWORD cbData = sizeof(DWORD);
   if (RegQueryValueEx( hKey, TEXT("DebugIME32"), NULL, NULL, (LPBYTE)&data, &cbData) != ERROR_SUCCESS) {
	   return 0;
   }
   return data;
}

inline void DebugOut(DWORD level, const WCHAR *s)
{
    if (GetDebugMode() >= level && s) {
        OutputDebugString(s);
    }
}

inline void DebugOut(DWORD level, const WCHAR *s, const WCHAR *t)
{
    if (GetDebugMode() >= level) {
        wstringstream stream;
        if (s) {
            stream << s;
        }
        if (t) {
            stream << t;
        }
        OutputDebugString(stream.str().c_str());
    }
}

inline void DebugOut(DWORD level, const WCHAR *s, LONG t)
{
    if (GetDebugMode() >= level) {
        wstringstream stream;
        if (s) {
            stream << s;
        }
        stream << t;
        OutputDebugString(stream.str().c_str());
    }
}

inline void DebugOutLastError(DWORD level)
{
    if (GetDebugMode() >= level) {
        WCHAR *lpMsgBuf;
        FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
            NULL,
            GetLastError(),
            MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR) &lpMsgBuf,
            0,
            NULL
            );
        OutputDebugString(lpMsgBuf);
    }
}

#endif
