﻿/*
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 Masataka Shinke, Takuya Nishimoto (nvdajp)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
# nvdajpime/nvdajpime.cpp
*/

#include <sstream>
#include "nvdajpime.h"
#include "tls.h"
#include "tsf.h"

using namespace std;

HMODULE g_hModule = 0;

Cnvdajpime *g_Cnvdajpime = NULL;

TLS g_TLS;
DWORD TLS::dwTLSIndex = TLS_OUT_OF_INDEXES;

CnvdajpimeSharedMem g_SharedMemory;

// static method
UINT WINAPI Cnvdajpime::Thread(void *p_this)
{
	Cnvdajpime *m_this = reinterpret_cast<Cnvdajpime *>(p_this);
	HANDLE hEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, EVENT_NAME);
	if (hEvent == NULL)
	{
		DebugOut(2, L"x86: OpenEvent FAILED");
		DebugOutLastError(1);
	}
	while (m_this->m_RunFlag)
	{
		DebugOut(4, L"x86: Thread waiting..");
		DWORD ret = WaitForSingleObject(hEvent, INFINITE);
		if (ret == WAIT_FAILED)
		{
			DebugOut(2, L"x86: WaitForSingleObject FAILED");
			DebugOutLastError(1);
		}
		ResetEvent(hEvent);
		if (GetSharedMemory()->CallbackAvailable)
		{
			if (m_this->m_CallBack != NULL)
			{
				m_this->m_CallBack(
					GetSharedMemory()->cursorPos,
					GetSharedMemory()->deltaStart,
					GetSharedMemory()->composition,
					GetSharedMemory()->compAttr,
					GetSharedMemory()->result,
					GetSharedMemory()->textService
					);
			}
			GetSharedMemory()->CallbackAvailable = FALSE;
		}
	}
	CloseHandle(hEvent);
	 
	TLS::DestroyTLS();
	return 0;
}


void SetEventToSignal()
{
	HANDLE hEvent = OpenEvent(EVENT_ALL_ACCESS, FALSE, EVENT_NAME);
	if (hEvent == NULL)
	{
		DebugOut(2, L"x86: OpenEvent FAILED");
		DebugOutLastError(1);
		return;
	}
	SetEvent(hEvent);													
	CloseHandle(hEvent);												
}

void UpdateIMMCompositionString()
{
	DebugOut(4, L"x86: UpdateIMMCompositionString()");

	DWORD dwRet = 0;
	HWND hwnd = GetFocus();
	if (!hwnd) {
		DebugOut(2, L"x86: UpdateIMMCompositionString no focus");
		return;
	}
	HWND hime = ImmGetDefaultIMEWnd(hwnd);
	if (!hime) {
		DebugOut(2, L"x86: UpdateIMMCompositionString no hime");
		return;
	}
	HIMC himc = ImmGetContext(hime);
	if (!himc) {
		DebugOut(2, L"x86: UpdateIMMCompositionString no himc");
		return;
	}
	if (himc) {
		// setting result string.
		//
		// ImmGetCompositionString() with GCS_RESULTSTR
		// returns the byte count of unicode string.
		// This value is available if an IME session is just finished.
		// If an IME session is not finished, or 
		// if new IME session is already started, this value is empty.
		WCHAR szResultStr[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_RESULTSTR, 
										 szResultStr, 
										 sizeof(szResultStr));
		szResultStr[dwRet/sizeof(WCHAR)] = L'\0';
		DebugOut(3, L"x86: result:", szResultStr);

		// setting composition string.
		//
		// ImmGetCompositionString() with GCS_COMPSTR
		// returns the byte count of unicode string.
		// Both pre-edit and composition string can be obtained.
		WCHAR szComposition[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_COMPSTR, 
										 szComposition, 
										 sizeof(szComposition));
		int compositionLen = (int)(dwRet/sizeof(WCHAR));
		szComposition[compositionLen] = L'\0';
		DebugOut(3, L"x86: composition:", szComposition);

		// setting cursorPos, cursorMoveStr and cursorAvailable.
		//
		// ImmGetCompositionString() with GCS_CURSORPOS
		// returns the position of caret.
		// Both under pre-edit mode and composition mode, 
		// pressing left-arrow key or right-arrow key moves the caret. 
		// Position 0 is the left boundary.
		// If the composition string consists of 3 characters,
		// position 3 is the right boundary.
		//
		// After pressing conversion key (i.e. when composition mode is started),
		// ATOK Japanese IME moves the caret position to 0, 
		// and Microsoft Japanese IME moves the caret to the right boundery, respectively.
		//
		// Here we obtain the left-adjacent charater of caret in Python code.
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_CURSORPOS, 
										 NULL, 0);
		int cursorPos = (int)dwRet;
		DebugOut(3, L"x86: cursorPos:", cursorPos);
		
		// setting delta string.
		//
		// ImmGetCompositionString() with GCS_DELTASTART
		// returns the character position of starting 'delta'
		// within pre-edit or composition string.
		// Delta string is used as the feedback to key down event.
		// If all of the characters within the compostion is updated
		// after the key input, the delta start is 0.
		// If only first character is not updated, delta start is 1.
		// 
		// We obtain delta string using delta start position in Python code.
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_DELTASTART, 
										 NULL, 0);
		DebugOut(4, L"x86: (DELTASTART):dwRet:", dwRet);
		int deltaStartPos = (int)dwRet;

		// setting selected string.
		//
		// ImmGetCompositionString() with GCS_COMPATTR
		// returns the byte count of array.
		// Arguments are the array of bytes and maximum array size as byte.
		// The result array items are attribute values for each characters as follows:
		//   0: not converted
		//   1: selected
		//   2: not selected
		//
		// As we observed, the characters within an clause have same attribute value.
		//
		// We obtain a string which consists of selected characters in Python code.
		BYTE szCompAttr[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_COMPATTR, 
										 szCompAttr, sizeof(szCompAttr));
		int compAttrCount = (int)dwRet;
		// setting compAttr string.
		// example: L"222221111000"
		WCHAR compAttr[STRLEN] = {0};
		for (int i = 0; i < compAttrCount; i++) {
			compAttr[i] = L'0' + szCompAttr[i];
		}
		compAttr[compAttrCount] = 0;
		DebugOut(3, L"x86: compAttr:", compAttr);

		// release context
		ImmReleaseContext(hwnd, himc);

		// send to Python
		NotifyInputMethod(cursorPos, deltaStartPos, szComposition, compAttr, szResultStr);
	}
}

void _UninitTextEdit()
{
	TLS *ptls = TLS::GetTLS();
	if (ptls && ptls->_pReadCompTSF)
	{
		ptls->_pReadCompTSF->_InitTextEditSink(NULL);
	}
}

UINT _SysGetMsgProc(WPARAM wParam, LPARAM lParam)
{
	MSG *pmsg;
	UINT uMsg;

	pmsg = (MSG *)lParam;
	uMsg = pmsg->message;

	switch (uMsg)
	{
	case WM_IME_STARTCOMPOSITION:
		DebugOut(4, L"x86: WM_IME_STARTCOMPOSITION");
		break;
	case WM_IME_ENDCOMPOSITION:
		DebugOut(4, L"x86: WM_IME_ENDCOMPOSITION");
		TLS::DestroyTLS();
		break;
	case WM_IME_COMPOSITION:
		UpdateIMMCompositionString();
		break;
	case WM_IME_NOTIFY:
		UpdateIMMCompositionString();
		break;
	case WM_KEYDOWN:
		DebugOut(4, L"x86: WM_KEYDOWN");
		CnvdajpimeTSF::_Init();
		break;
	case WM_KEYUP:
		DebugOut(4, L"x86: WM_KEYUP");
		CnvdajpimeTSF::_Init();
		break;
	}
	return 1;
}

LRESULT CALLBACK SysGetMsgProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	HHOOK hHook;

	hHook = GetSharedMemory()->hSysGetMsgHook;

	if (nCode == HC_ACTION && (wParam & PM_REMOVE))
	{
		_SysGetMsgProc(wParam, lParam);
	}
	return CallNextHookEx(hHook, nCode, wParam, lParam);
}

Cnvdajpime::Cnvdajpime()
{
	HANDLE hEvent = CreateEvent(NULL, TRUE, FALSE, EVENT_NAME);
	if (hEvent == NULL)
	{
		DebugOut(2, L"x86: CreateEvent FAILED");
		DebugOutLastError(1);
	}
	else
	{
		DebugOut(2, L"x86: Cnvdajpime() CreateEvent done.");
	}

	HHOOK hSysGetMsgHook = SetWindowsHookEx(WH_GETMESSAGE, 
											SysGetMsgProc, 
											g_hModule, 
											0);
	if (!hSysGetMsgHook)
	{
		DebugOut(2, L"x86: Cnvdajpime() constructor SetWindowsHookEx(WH_GETMESSAGE) failed.");
		DebugOutLastError(1);
		return;
	}
	GetSharedMemory()->hSysGetMsgHook = hSysGetMsgHook;

	GetSharedMemory()->fReadCompRunning = TRUE;

	GetSharedMemory()->cursorPos = 0;
	GetSharedMemory()->deltaStart = 0;
	GetSharedMemory()->composition[0] = 0;
	GetSharedMemory()->compAttr[0] = 0;
	GetSharedMemory()->result[0] = 0;
	GetSharedMemory()->textService[0] = 0;

	this->m_RunFlag = true;

	DebugOut(2, L"x86: Cnvdajpime() constructed.");
}

Cnvdajpime::~Cnvdajpime()
{
	DebugOut(2, L"x86: Cnvdajpime() destructed.");
}

extern "C" NVDAJPIME_API BOOL WINAPI Initialize(CallBackProc p_CallBack)
{
	DebugOut(2, L"x86: nvdajpime/nvdajpime.cpp Initialize() starting.");

	g_Cnvdajpime = new Cnvdajpime();

	if (p_CallBack != NULL)
	{
		g_Cnvdajpime->m_CallBack = p_CallBack;
		DebugOut(2, L"x86: nvdajpime/nvdajpime.cpp Initialize() CB set");
	}
	else
	{
		g_Cnvdajpime->m_CallBack = NULL;
	} 

	UINT_PTR h = _beginthreadex(NULL, 0, g_Cnvdajpime->Thread, g_Cnvdajpime, 0, NULL);
	if (h == NULL)
	{
		DebugOut(2, L"x86: _beginthreadex() FAILED");
		DebugOutLastError(1);
	}
	else
	{
		DebugOut(2, L"x86: _beginthreadex() done.");
	}
	g_Cnvdajpime->m_Thread = (HANDLE)h;
	GetSharedMemory()->fReadCompRunning = TRUE;
	GetSharedMemory()->CallbackAvailable = FALSE;

	SetEventToSignal();
	
	DebugOut(2, L"x86: nvdajpime/nvdajpime.cpp Initialize() done.");
	return TRUE; 
}

extern "C" NVDAJPIME_API void WINAPI Terminate(void)
{
	DebugOut(2, L"x86: nvdajpime/nvdajpime.cpp Terminate() starting.");
	g_Cnvdajpime->m_RunFlag = false;

	SetEventToSignal();

	DWORD ret = WaitForSingleObject( g_Cnvdajpime->m_Thread, INFINITE );
	if (ret == WAIT_FAILED)
		DebugOut(2, L"x86: WaitForSingleObject FAILED");
	CloseHandle( g_Cnvdajpime->m_Thread );
	delete g_Cnvdajpime;
	g_Cnvdajpime = NULL;
	 
	GetSharedMemory()->fReadCompRunning = FALSE;

	HHOOK hSysGetMsgHook = GetSharedMemory()->hSysGetMsgHook;
	UnhookWindowsHookEx(hSysGetMsgHook);

	TLS::DestroyTLS();
	DebugOut(2, L"x86: nvdajpime/nvdajpime.cpp Terminate() done.");
}

BOOL NotifyTextService(WCHAR *buff)
{
	GetSharedMemory()->cursorPos = 0;
	GetSharedMemory()->deltaStart = 0;
	GetSharedMemory()->composition[0] = 0;
	GetSharedMemory()->compAttr[0] = 0;
	GetSharedMemory()->result[0] = 0;
	wcscpy_s(GetSharedMemory()->textService, STRLEN, buff);
	GetSharedMemory()->CallbackAvailable = TRUE;
	SetEventToSignal();
	return TRUE;
}

BOOL NotifyInputMethod(UINT cursorPos, UINT deltaStart, WCHAR *composition, WCHAR *compAttr, WCHAR *result)
{
	GetSharedMemory()->cursorPos = cursorPos;
	GetSharedMemory()->deltaStart = deltaStart;
	wcscpy_s(GetSharedMemory()->composition, STRLEN, composition);
	wcscpy_s(GetSharedMemory()->compAttr, STRLEN, compAttr);
	wcscpy_s(GetSharedMemory()->result, STRLEN, result);
	GetSharedMemory()->textService[0] = 0;
	GetSharedMemory()->CallbackAvailable = TRUE;
	SetEventToSignal();
	return TRUE;
}
