﻿/*
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 Masataka Shinke, Takuya Nishimoto (nvdajp)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#
# nvdajpime/nvdajpime.h
*/

#define DEBUGOUT  1

#ifdef NVDAJPIME_EXPORTS
#define NVDAJPIME_API __declspec(dllexport)
#else
#define NVDAJPIME_API __declspec(dllimport)
#endif

#ifdef TARGET_64
#define EVENT_NAME TEXT("Event Object nvdajpime3_64")
#define SHAREDM TEXT("nvdajpimeSharedMem_64")
#else
#define EVENT_NAME TEXT("Event Object nvdajpime3_32")
#define SHAREDM TEXT("nvdajpimeSharedMem_32")
#endif

#ifndef _WIN32_WINNT   
#define _WIN32_WINNT 0x0600
#endif

#ifndef _WIN32_WINDOWS
#define _WIN32_WINDOWS 0x0410
#endif

#ifndef _WIN32_IE 
#define _WIN32_IE 0x0700
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <process.h>
#include <stdlib.h>
#include <msctf.h>
#include <string>
#include <assert.h>
#include <sstream>
#include "debugout.h"

typedef void (WINAPI *CallBackProc)(UINT cursorPos, UINT deltaStart, WCHAR *composition, WCHAR *compAttr, WCHAR *result, WCHAR *textService);

class Cnvdajpime {
public:
	Cnvdajpime(void);
	virtual ~Cnvdajpime(void);

	CallBackProc	m_CallBack;
	HANDLE			m_Thread;
	BOOL			m_RunFlag;
	static UINT WINAPI Thread(void *p);
};

extern HMODULE				g_hModule;
extern Cnvdajpime           *g_Cnvdajpime;

class	TLS; 
extern	TLS					g_TLS;

class CReadCompFileMapping
{
public:
    void BaseInit()
    {
        _pv = NULL;
        _hfm = NULL;
    }

    void Init(TCHAR *pszFile)
    {
        _pszFile = pszFile;
        _fCreated = FALSE;
    }

    void Uninit()
    {
    }

    void *Open()
    {
        _hfm = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, _pszFile);

        if (_hfm == NULL)
            return NULL;

        return _Map();
    }

    void *Create(SECURITY_ATTRIBUTES *psa, ULONG cbSize, BOOL *pfAlreadyExists)
    {
        _hfm = CreateFileMapping(INVALID_HANDLE_VALUE, psa, PAGE_READWRITE,
                                 0, cbSize, _pszFile);

        if (pfAlreadyExists != NULL)
        {
            *pfAlreadyExists = (GetLastError() == ERROR_ALREADY_EXISTS);
        }

        if (_hfm == NULL)
            return NULL;

        _fCreated = TRUE;
        return _Map();
    }

    BOOL Flush(UINT cbSize)
    {
        if (!_pv)
            return FALSE;

        return FlushViewOfFile(_pv, cbSize);
    }

    void Close()
    {
        if (_pv)
            UnmapViewOfFile(_pv);

        if (_hfm)
            CloseHandle(_hfm);

        _pv = NULL;
        _hfm = NULL;
        _fCreated = FALSE;
    }

    void SetName(TCHAR *psz) {_pszFile = psz;}


    BOOL IsCreated() { return _fCreated; }

private:
    void *_Map()
    {
        _pv = (void *)MapViewOfFile(_hfm, FILE_MAP_WRITE, 0, 0, 0);
        if (!_pv)
        {
            CloseHandle(_hfm);
            _hfm = NULL;
        }
        return _pv;
    }

protected:
    TCHAR *_pszFile;
    void *_pv;
private:
    HANDLE _hfm;
    BOOL _fCreated;
};

const int STRLEN = 256;

typedef struct
{
	HHOOK hSysGetMsgHook;
	BOOL  fReadCompRunning;
  
	UINT	cursorPos;
	UINT	deltaStart;
	WCHAR	composition[STRLEN];
	WCHAR	compAttr[STRLEN];
	WCHAR	result[STRLEN];
	WCHAR	textService[STRLEN];

	BOOL		RunFlag;				
	HANDLE	    hThread;				
	UINT		threadID;				
	
	BOOL		CallbackAvailable;
} SHAREDMEM;

class CnvdajpimeSharedMem : public CReadCompFileMapping
{
public:
    BOOL Start()
    {
        DebugOut(4, L"x86: CnvdajpimeSharedMem::Start() called.");
        BOOL fAlreadyExists = FALSE;

        BaseInit();
        Init(SHAREDM);

        if (Create(NULL, sizeof(SHAREDMEM), &fAlreadyExists) == NULL)
        {
            DebugOut(2, L"x86: CnvdajpimeSharedMem::Start() Create() returned NULL");
            return FALSE;
        }
        if (!fAlreadyExists)
        {
            DebugOut(2, L"x86: CnvdajpimeSharedMem::Start() AlreadyExists");
        }
        DebugOut(4, L"x86: CnvdajpimeSharedMem::Start() finished.");
        return TRUE;
    }

    SHAREDMEM *GetPtr() { return (SHAREDMEM *)_pv; }

    ~CnvdajpimeSharedMem()
    {
        extern CnvdajpimeSharedMem g_SharedMemory;
        g_SharedMemory.Flush(0);
        DebugOut(4, L"x86: CnvdajpimeSharedMem destructed.");
    }
private:
};

extern CnvdajpimeSharedMem g_SharedMemory;

inline SHAREDMEM *GetSharedMemory() { return g_SharedMemory.GetPtr(); }
inline void FlushSharedMemory() { g_SharedMemory.Flush(0); }

BOOL NotifyTextService(WCHAR *buff);
BOOL NotifyInputMethod(UINT cursorPos, UINT deltaStart, WCHAR *composition, WCHAR *compAttr, WCHAR *result);
DWORD GetDebugMode();

