﻿/*
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 Masataka Shinke, Takuya Nishimoto (nvdajp)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#
# nvdajpimeRPC/nvdajpime.cpp
# 
*/

#include "nvdajpime.h"
#include "tls.h"
#include "tsf.h"
#include "debugout.h"

void * __RPC_USER midl_user_allocate(size_t size) 
{
	return malloc(size);
}

void __RPC_USER midl_user_free(void* p) 
{
	free(p);
}

HMODULE	g_hModule;

TLS	g_TLS;
DWORD TLS::dwTLSIndex = TLS_OUT_OF_INDEXES;

CnvdajpimeSharedMem	g_SharedMemory;

void RPC_Bind()
{
	wchar_t desktopSpecificNamespace[64];
	generateDesktopSpecificNamespace(desktopSpecificNamespace, ARRAYSIZE(desktopSpecificNamespace));
	wstringstream s;
	s << L"ncalrpc:[NvdaCtlr." << desktopSpecificNamespace << L"]";
	RpcBindingFromStringBinding((RPC_WSTR)(s.str().c_str()), &nvdaControllerBindingHandle);
}

void RPC_Free()
{
	RpcBindingFree(&nvdaControllerBindingHandle);
}

BOOL GetIMMOpenStatus()
{
    HWND hwnd = NULL;
    HIMC himc = NULL;
	BOOL Ret = FALSE;

    hwnd = GetFocus();
	if (!hwnd) {
		DebugOut(2, L"RPC: GetIMMOpenStatus: GetFocus returns NULL");
		return FALSE; // return GetSharedMemory()->ImeOpenStatus;
	}

    himc = ImmGetContext(hwnd);
    if (himc)
    {
		Ret = ImmGetOpenStatus(himc);
        ImmReleaseContext(hwnd, himc);
    }
	return Ret;
}

void UpdateIMMCompositionString()
{
	const int STRLEN = 256;
	DWORD dwRet = 0;
	HWND hwnd = GetFocus();
	if (!hwnd) {
		return;
	}
	HWND hime = ImmGetDefaultIMEWnd(hwnd);
	if (!hime) {
		return;
	}
	HIMC himc = ImmGetContext(hime);
	if (!himc) {
		return;
	}
	if (himc) {
		// see comments in nvdaHelper/nvdajpime/nvdajpime.cpp
		WCHAR szResultStr[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_RESULTSTR, 
										 szResultStr, 
										 sizeof(szResultStr));
		szResultStr[dwRet/sizeof(WCHAR)] = L'\0';

		WCHAR szComposition[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_COMPSTR, 
										 szComposition, 
										 sizeof(szComposition));
		int compositionLen = (int)(dwRet/sizeof(WCHAR));
		szComposition[compositionLen] = L'\0';

		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_CURSORPOS, 
										 NULL, 0);
		int cursorPos = (int)dwRet;

		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_DELTASTART, 
										 NULL, 0);
		int deltaStartPos = (int)dwRet;

		BYTE szCompAttr[STRLEN] = {0};
		dwRet = ImmGetCompositionStringW(himc, 
										 GCS_COMPATTR, 
										 szCompAttr, sizeof(szCompAttr));
		int compAttrCount = (int)dwRet;
		// setting compAttr string.
		// example: L"222221111000"
		WCHAR compAttr[STRLEN] = {0};
		for (int i = 0; i < compAttrCount; i++) {
			compAttr[i] = L'0' + szCompAttr[i];
		}
		compAttr[compAttrCount] = 0;

		// release context
		ImmReleaseContext(hwnd, himc);

		// send to Python
		if (nvdajpimeRPC_testIfRunning()) {
			RPC_Bind();
			const wchar_t* textService = L"";
			nvdajpimeRPC_inputMethodCallback(cursorPos, deltaStartPos, szComposition, compAttr, szResultStr, textService);
			RPC_Free();
		}
	}	
}

void _UninitTextEdit()
{
    TLS *ptls = TLS::GetTLS();
    if (ptls && ptls->_pReadCompTSF)
    {
        ptls->_pReadCompTSF->_InitTextEditSink(NULL);
    }
}

UINT _SysGetMsgProc(WPARAM wParam, LPARAM lParam)
{
    MSG *pmsg;
    UINT uMsg;

    pmsg = (MSG *)lParam;
    uMsg = pmsg->message;

    switch (uMsg)
    {
	case WM_IME_STARTCOMPOSITION:
		DebugOut(4, L"RPC: WM_IME_STARTCOMPOSITION");
		break;
	case WM_IME_ENDCOMPOSITION:
		DebugOut(4, L"RPC: WM_IME_ENDCOMPOSITION");
		TLS::DestroyTLS();
		break;
	case WM_IME_COMPOSITION:
		UpdateIMMCompositionString();
		break;
	case WM_IME_NOTIFY:
		UpdateIMMCompositionString();
		break;
	case WM_KEYDOWN:
		CnvdajpimeTSF::_Init();
		break;
	case WM_KEYUP:
		CnvdajpimeTSF::_Init();
		break;
    }
    return 1;
}

LRESULT CALLBACK SysGetMsgProc(int nCode, WPARAM wParam, LPARAM lParam)
{
    HHOOK hHook;

    hHook = GetSharedMemory()->hSysGetMsgHook;

    if (nCode == HC_ACTION && (wParam & PM_REMOVE))
    {
        _SysGetMsgProc(wParam, lParam);
    }
    return CallNextHookEx(hHook, nCode, wParam, lParam);
}

extern "C" NVDAJPIME_API BOOL WINAPI Initialize(void)
{
	DebugOut(2, L"RPC: Initialize() starting.");
    HHOOK hSysGetMsgHook = SetWindowsHookEx(
		WH_GETMESSAGE, 
		SysGetMsgProc, 
		g_hModule, 
		0);
    if (!hSysGetMsgHook)
        return FALSE;
    GetSharedMemory()->hSysGetMsgHook = hSysGetMsgHook;
	GetSharedMemory()->fReadCompRunning = TRUE;
	return TRUE; 
}

extern "C" NVDAJPIME_API BOOL WINAPI Terminate(void)
{
	GetSharedMemory()->fReadCompRunning = FALSE;

	HHOOK hSysGetMsgHook = GetSharedMemory()->hSysGetMsgHook;
	UnhookWindowsHookEx(hSysGetMsgHook);
	TLS::DestroyTLS();
	DebugOut(2, L"RPC: Terminate() done.");
	return TRUE;
}
