﻿/*
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2010-2012 Masataka Shinke, Takuya Nishimoto (nvdajp)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#
# nvdajpimeRPC/tls.h
*/

#ifndef _TLS_H_
#define _TLS_H_

#include "debugout.h"

class CnvdajpimeTSF;

class TLS
{
public:
    static inline BOOL Initialize()
    {
        dwTLSIndex = TlsAlloc();
        if (dwTLSIndex == TLS_OUT_OF_INDEXES)
            return FALSE;

        return TRUE;
    }

    static inline void Uninitialize()
    {
        if (dwTLSIndex != TLS_OUT_OF_INDEXES)
        {
            TlsFree(dwTLSIndex);
            dwTLSIndex = TLS_OUT_OF_INDEXES;
        }
    }

    static inline TLS *GetTLS()
    {
        return InternalAllocateTLS();
    }

    static inline TLS *ReferenceTLS()
    {
        return (TLS *)TlsGetValue(dwTLSIndex);
    }

    static inline BOOL DestroyTLS()
    {
        DebugOut(2, L"RPC: TLS::DestroyTLS() called.");
        return InternalDestroyTLS();
    }

private:
    static inline TLS *InternalAllocateTLS()
    {
        TLS *ptls = (TLS *)TlsGetValue(dwTLSIndex);
        if (ptls == NULL)
        {
            if ((ptls = (TLS *)LocalAlloc(LPTR, sizeof(TLS))) == NULL)
                return NULL;

            if (! TlsSetValue(dwTLSIndex, ptls))
            {
                LocalFree(ptls);
                return NULL;
            }
            ptls->_pReadCompTSF = NULL;
            DebugOut(2, L"RPC: TLS::InternalAllocateTLS() allocated.");
        }
        return ptls;
    }

    static BOOL InternalDestroyTLS();

private:
    static DWORD dwTLSIndex;

public:
    CnvdajpimeTSF *_pReadCompTSF;
};

#endif // _TLS_H_
