# coding: UTF-8
#brailleDisplayDrivers/_kgs.py
#A part of NonVisual Desktop Access (NVDA)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#Copyright (C) 2011-2012 Takuya Nishimoto, Masataka Shinke
#
# supported device: Braille Memo BM46
# testing:
# > cd betterBraille\source
# > python brailleDisplayDrivers\_kgs.py

import time
import os
from ctypes import *
from ctypes.wintypes import *

try:
	import hwPortUtils
except:
	import os
	import sys
	sys.path.append(os.getcwdu())
	import hwPortUtils

if __name__ == '__main__':
	class Logger(object):
		def debug(self, s):
			print s
		def info(self, s):
			print s
	log = Logger()
	class Watchdog(object):
		def alive(self):
			pass
	watchdog = Watchdog()
	def beep(t = 0):
		pass
else:
	from logHandler import log
	import watchdog
	def beep(t = 1):
		# use try-except because tones.beep may cause error on terminating NVDA
		try:
			import tones
			if   t == 1: # connecting tick
				tones.beep(500, 20)
			elif t == 2: # connect success
				tones.beep(1000, 30)
			elif t == 3: # error
				tones.beep(200, 500)
			elif t == 4: # disconnecting tick
				tones.beep(200, 20)
			elif t == 5: # disconnect success
				tones.beep(1000, 500)
			elif t == 6: # port release success
				tones.beep(1500, 500)
			elif t == 7: # other event
				tones.beep(2000, 30)
		except:
			pass

BMDRVS_DISCONNECTED = 0
BMDRVS_CONNECTED = 1
BMDRVS_DRIVER_CANNOT_OPEN = 2
BMDRVS_INVALID_DRIVER = 3
BMDRVS_OPEN_PORT_FAILED = 4
BMDRVS_CREATE_THREAD_FAILED = 5
BMDRVS_CHECKING_EQUIPMENT = 6
BMDRVS_UNKNOWN_EQUIPMENT = 7
BMDRVS_PORT_RELEASED = 8
BMDRVS_MAX = 9

BM_DISPMODE_FOREGROUND = 0x01
BM_DISPMODE_BACKGROUND = 0x02
BM_DISPMODE_KEYHANDLER = 0x04
BM_DISPMODE_SUSPENDED  = 0x08
DISPMODE = BM_DISPMODE_BACKGROUND

REGKEY_KBDC110_PATH_JA = "SOFTWARE\\KGS\\KBDC110"
DEVNAME_JA = u"BMシリーズ機器".encode('shift-jis')
REGKEY_KBDC110_PATH_EN = "SOFTWARE\\KGS\\KBDC110-E"
DEVNAME_EN = "BM series"

SPEED = 3 # 9600bps
OWNERNAME = 'NVDA'

kgsDLL = None
connected = False
numCells = 0
regkeyEn = False
handleGestureFunc = None

# void (CALLBACK *pStatusChanged)(int nStatus, int nDispSize)
@WINFUNCTYPE(c_void_p, c_int, c_int)
def statusChanged(nStatus, nDispSize):
	#log.debug("statusChanged")
	global connected, numCells
	if nStatus == BMDRVS_CONNECTED:
		log.info("connected size:%d" % nDispSize)
		numCells = nDispSize
		connected = True
		beep(2)
	elif nStatus == BMDRVS_DISCONNECTED:
		log.info("disconnected")
		connected = False
		beep(5)
	elif nStatus == BMDRVS_PORT_RELEASED:
		log.info("port released")
		beep(6)
	elif nStatus == BMDRVS_CHECKING_EQUIPMENT:
		log.info("checking equipment")
		beep(7)
	elif nStatus == BMDRVS_UNKNOWN_EQUIPMENT:
		log.info("checking equipment")
		beep(7)
	else:
		log.info("status changed to %d" % nStatus)

# BOOL (CALLBACK *pHandleKeyInfo)(BYTE info[4])
@WINFUNCTYPE(c_int, POINTER(c_ubyte))
def handleKeyInfo(lpKeys):
	keys = (lpKeys[0], lpKeys[1], lpKeys[2], lpKeys[3])
	log.debug("keyInfo %d %d %d %d" % keys)
	names = set()
	routingIndex = None
	if keys[2] &   1: names.add('func1')
	if keys[2] &   2: names.add('func4')
	if keys[2] &   4: names.add('ctrl')
	if keys[2] &   8: names.add('alt')
	if keys[2] &  16: names.add('select')
	if keys[2] &  32: names.add('read')
	if keys[2] &  64: names.add('func2')
	if keys[2] & 128: names.add('func3')
	if keys[0] == 1:
		if keys[1] &   1: names.add('space')
		if keys[1] &   2: names.add('dot6')
		if keys[1] &   4: names.add('dot5')
		if keys[1] &   8: names.add('dot4')
		if keys[1] &  16: names.add('enter')
		if keys[1] &  32: names.add('dot3')
		if keys[1] &  64: names.add('dot2')
		if keys[1] & 128: names.add('dot1')
	elif keys[0] == 2:
		if keys[1] &   1: names.add('esc')
		if keys[1] &   2: names.add('inf')
		if keys[1] &   4: names.add('bs')
		if keys[1] &   8: names.add('del')
		if keys[1] &  16: names.add('ins')
		if keys[1] &  32: names.add('chng')
		if keys[1] &  64: names.add('ok')
		if keys[1] & 128: names.add('set')
	elif keys[0] == 3:
		if keys[1] & 1: names.add('upArrow')
		if keys[1] & 2: names.add('downArrow')
		if keys[1] & 4: names.add('leftArrow')
		if keys[1] & 8: names.add('rightArrow')
	elif keys[0] == 4:
		names.add('route')
		routingIndex = keys[1]
	elif keys[0] == 6:
		if keys[1] & 1: names.add('bw')
		if keys[1] & 2: names.add('fw')
		if keys[1] & 4: names.add('ls')
		if keys[1] & 8: names.add('rs')
	if handleGestureFunc: handleGestureFunc(names, routingIndex)
	if routingIndex:
		log.debug("names %s %d" % ('+'.join(names), routingIndex))
	else:
		log.debug("names %s" % '+'.join(names))
	if len(names): return True
	return False

def load():
	global kgsDLL, regkeyEn
	if not kgsDLL:
		kgsDLL = windll.LoadLibrary("lib/DirectBM.dll")
		log.debug("LoadLibrary done %s" % str(kgsDLL))
	ret = kgsDLL.IsKbdcInstalled(REGKEY_KBDC110_PATH_JA)
	log.debug("IsKbdcInstalled(ja) %d" % ret)
	if not ret:
		ret = kgsDLL.IsKbdcInstalled(REGKEY_KBDC110_PATH_EN)
		if ret:
			log.debug("IsKbdcInstalled(en) %d" % ret)
			regkeyEn = True
	return ret

def connect(comPort = None):
	log.debug("scanning port")
	if regkeyEn:
		devname = DEVNAME_EN
	else:
		devname = DEVNAME_JA
	if comPort is not None:
		if os.environ.get('PROCESSOR_ARCHITEW6432') == 'AMD64':
			ret = kgsDLL.bmStart(devname, comPort, SPEED, statusChanged)
			log.info("bmStart %d" % ret)
		else:
			ret = kgsDLL.bmStartInProcess(OWNERNAME, devname, comPort, SPEED, statusChanged, False)
			log.info("bmStartInProcess %d" % ret)
		if ret:
			for loop in xrange(10):
				beep(1)
				time.sleep(1.0)
				watchdog.alive()
				if connected:
					log.info("connected comPort {p}".format(p=comPort))
					beep(2)
					_startDisplayMode()
					return True
		beep(3)
		disconnect()
		return False
	for portInfo in hwPortUtils.listComPorts(onlyAvailable=True):
		portName = portInfo["port"]
		hwID = portInfo["hardwareID"]
		fName = portInfo.get("frendlyName")
		btName = portInfo.get("bluetoothName")
		log.info("scanning port:{portName} hw:{hwID} name:{fName} bt:{btName}".format(portName=portName, hwID=hwID, btName=btName, fName=fName))
		if hwID[:3] != 'USB':
			continue
		if portName[:3] == 'COM':
			port = int(portName[3:])
			if os.environ.get('PROCESSOR_ARCHITEW6432') == 'AMD64':
				ret = kgsDLL.bmStart(devname, port - 1, SPEED, statusChanged)
				log.info("bmStart %d" % ret)
 			else:
				ret = kgsDLL.bmStartInProcess(OWNERNAME, devname, port - 1, SPEED, statusChanged, False)
				log.info("bmStartInProcess %d" % ret)
			if ret:
				for loop in xrange(20):
					beep(1)
					time.sleep(0.5)
					watchdog.alive()
					if connected:
						log.info("connected {p}".format(p=portName))
						beep(2)
						_startDisplayMode()
						return True
	log.info("connect failed")
	beep(3)
	disconnect()
	return False

def _startDisplayMode():
	ret = kgsDLL.bmStartDisplayMode2(DISPMODE, handleKeyInfo)
	log.debug("bmStartDisplayMode2 %d" % ret)
	return ret

def setGestureHandler(func):
	global handleGestureFunc
	handleGestureFunc = func

def disconnect():
	global connected, numCells
	log.info("begin disconnect")
	if not kgsDLL: return
	ret = kgsDLL.bmEndDisplayMode()
	log.info("bmEndDisplayMode %d" % ret)
	time.sleep(0.5)
	ret = kgsDLL.bmEnd()
	log.info("bmEnd %d" % ret)
	for loop in xrange(20):
		beep(4)
		log.info("wait for disconnect")
		time.sleep(0.5)
		watchdog.alive()
		if not connected:
			log.info("status changed to disconnected")
			break
	connected = False
	log.info("end disconnect")
	return

def unload():
	global kgsDLL, handleGestureFunc
	kgsDLL = None
	handleGestureFunc = None

def display(data):
	if not kgsDLL: return
	if not connected: return
	if not data or len(data) == 0: return
	s = ''
	for c in data:
		d = 0
		if c & 0x01: d += 0x80
		if c & 0x02: d += 0x40
		if c & 0x04: d += 0x20
		if c & 0x08: d += 0x08
		if c & 0x10: d += 0x04
		if c & 0x20: d += 0x02
		if c & 0x40: d += 0x10
		if c & 0x80: d += 0x01
		s += chr(d)
	dataBuf   = create_string_buffer(s, 256)
	cursorBuf = create_string_buffer('', 256)
	try:
		ret = kgsDLL.bmDisplayData(dataBuf, cursorBuf, numCells)
		log.debug("bmDisplayData %d" % ret)
	except:
		log.debug("error bmDisplayData")

if __name__ == '__main__':
	load()
	connect()
	display([0x20, 0x1d, 0x20, 0x27, 0x20, 0x19, 0x20, 0x01, 0x0d, 0x1d, 0x25]) # NVDAmnu
	time.sleep(20)
	unload()
