# coding: UTF-8
#brailleDisplayDrivers/kgs.py
#A part of NonVisual Desktop Access (NVDA)
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.
#Copyright (C) 2011-2012 Takuya Nishimoto, Masataka Shinke

import braille
import ui
import queueHandler
import inputCore
from logHandler import log
import config
import _kgs

def _exec(names, routingIndex):
	try:
		inputCore.manager.executeGesture(InputGesture(names, routingIndex))
	except:
		pass

class BrailleDisplayDriver(braille.BrailleDisplayDriver):
	name = "kgs"
	description = _(u"KGS device")
	initialized = False

	@classmethod
	def check(cls):
		return False

	def __init__(self, port="auto"):
		super(BrailleDisplayDriver, self).__init__()
		if self.initialized:
			log.info("unloading")
			_kgs.unload()
			self.initialized = False
		if _kgs.load():
			if self.supportComPortSelect:
				comPort = config.conf["braille"]["nvdajpComPort"]
			else:
				comPort = None
			if _kgs.connect(comPort):
				self.initialized = True
				_kgs.setGestureHandler(lambda names, routingIndex: _exec(names, routingIndex))
				log.info("connected")
				return
		log.info("failed")

	def terminate(self):
		if self.initialized:
			_kgs.unload()
			self.initialized = False
		super(BrailleDisplayDriver, self).terminate()
		log.info("terminated")

	def _get_numCells(self):
		return _kgs.numCells
	
	def display(self, cells):
		log.debug(" ".join(("%02x" % c) for c in cells))
		_kgs.display(cells)

	gestureMap = inputCore.GlobalGestureMap({
		"globalCommands.GlobalCommands": {
			"kb:alt": ("br(kgs):alt",),
			"kb:escape": ("br(kgs):esc",),
			"kb:delete": ("br(kgs):del",),
			"kb:backspace": ("br(kgs):bs",),
			"kb:space": ("br(kgs):space",),
			"kb:enter": ("br(kgs):enter","br(kgs):ok","br(kgs):set","br(kgs):select",),
			"kb:control": ("br(kgs):ctrl",),
			"kb:upArrow": ("br(kgs):upArrow",),
			"kb:downArrow": ("br(kgs):downArrow",),
			"kb:leftArrow": ("br(kgs):leftArrow",),
			"kb:rightArrow": ("br(kgs):rightArrow",),
			"braille_routeTo": ("br(kgs):route",),
			"braille_scrollBack": ("br(kgs):func1","br(kgs):func3+leftArrow",),
			"braille_scrollForward": ("br(kgs):func4","br(kgs):func3+rightArrow",),
			"braille_previousLine": ("br(kgs):func3+upArrow",),
			"braille_nextLine": ("br(kgs):func3+downArrow",),
			"showGui": ("br(kgs):ins",),
			"sayAll": ("br(kgs):read",),
			"kb:a": ("br(kgs):dot1",),
			"kb:b": ("br(kgs):dot1+dot2",),
			"kb:c": ("br(kgs):dot1+dot4",),
			"kb:d": ("br(kgs):dot1+dot4+dot5",),
			"kb:e": ("br(kgs):dot1+dot5",),
			"kb:f": ("br(kgs):dot1+dot2+dot4",),
			"kb:g": ("br(kgs):dot1+dot2+dot4+dot5",),
			"kb:h": ("br(kgs):dot1+dot2+dot5",),
			"kb:i": ("br(kgs):dot2+dot4",),
			"kb:j": ("br(kgs):dot2+dot4+dot5",),
			"kb:k": ("br(kgs):dot1+dot3",),
			"kb:l": ("br(kgs):dot1+dot2+dot3",),
			"kb:m": ("br(kgs):dot1+dot3+dot4",),
			"kb:n": ("br(kgs):dot1+dot3+dot4+dot5",),
			"kb:o": ("br(kgs):dot1+dot3+dot5",),
			"kb:p": ("br(kgs):dot1+dot2+dot3+dot4",),
			"kb:q": ("br(kgs):dot1+dot2+dot3+dot4+dot5",),
			"kb:r": ("br(kgs):dot1+dot2+dot3+dot5",),
			"kb:s": ("br(kgs):dot2+dot3+dot4",),
			"kb:t": ("br(kgs):dot2+dot3+dot4+dot5",),
			"kb:u": ("br(kgs):dot1+dot3+dot6",),
			"kb:v": ("br(kgs):dot1+dot2+dot3+dot6",),
			"kb:w": ("br(kgs):dot2+dot4+dot5+dot6",),
			"kb:x": ("br(kgs):dot1+dot3+dot4+dot6",),
			"kb:y": ("br(kgs):dot1+dot3+dot4+dot5+dot6",),
			"kb:z": ("br(kgs):dot1+dot3+dot5+dot6",),
			"kb:.": ("br(kgs):dot2+dot5+dot6",),
			"kb::": ("br(kgs):dot2+dot5",),
			"kb:;": ("br(kgs):dot2+dot3",),
			"kb:,": ("br(kgs):dot2",),
			"kb:-": ("br(kgs):dot3+dot6",),
			"kb:?": ("br(kgs):dot2+dot3+dot6",),
			"kb:!": ("br(kgs):dot2+dot3+dot5",),
			"kb:'": ("br(kgs):dot3",),
		}
	})

class InputGesture(braille.BrailleDisplayGesture):

	source = BrailleDisplayDriver.name

	def __init__(self, names, routingIndex):
		super(InputGesture, self).__init__()
		self.id = "+".join(names)
		self.routingIndex = routingIndex
